@@ -1,3 +1,20 @@
+//ChannelAttribution: Markov model for online multi-channel attribution
+//Copyright (C) 2015 - 2020  Davide Altomare and David Loris <http://www.channelattribution.net>
+//
+//ChannelAttribution is free software: you can redistribute it and/or modify
+//it under the terms of the GNU General Public License as published by
+//the Free Software Foundation, either version 3 of the License, or
+//(at your option) any later version.
+//
+//ChannelAttribution is distributed in the hope that it will be useful,
+//but WITHOUT ANY WARRANTY; without even the implied warranty of
+//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//GNU General Public License for more details.
+//
+//You should have received a copy of the GNU General Public License
+//along with ChannelAttribution.  If not, see <http://www.gnu.org/licenses/>.
+
+
 #define __GXX_EXPERIMENTAL_CXX0X__ 1
 
 #include <iostream>
@@ -419,651 +436,6 @@ vector<long int> split_string(const string &s, unsigned long int order) {
 	// }
 // }
 
-RcppExport SEXP markov_model_cpp(SEXP Data_p, SEXP var_path_p, SEXP var_conv_p, SEXP var_value_p, SEXP var_null_p, SEXP order_p, SEXP nsim_p, SEXP max_step_p, SEXP out_more_p, SEXP sep_p, SEXP seed_p)
-{
-	 	 
- BEGIN_RCPP
-
- //inp.a
-  
- List Data(Data_p);
- 
- CharacterVector var_path_0(var_path_p);
- string var_path = Rcpp::as<string>(var_path_0);
- 
- CharacterVector var_conv_0(var_conv_p);
- string var_conv = Rcpp::as<string>(var_conv_0);
- 
- CharacterVector var_value_0(var_value_p); 
- string var_value = Rcpp::as<string>(var_value_0);
- 
- CharacterVector var_null_0(var_null_p); 
- string var_null = Rcpp::as<string>(var_null_0);
- 
- NumericVector order_0(order_p); 
- unsigned long int order = Rcpp::as<unsigned long int>(order_0);
- 
- NumericVector nsim_0(nsim_p); 
- unsigned long int nsim = Rcpp::as<unsigned long int>(nsim_0);
-
- NumericVector max_step_0(max_step_p); 
- unsigned long int max_step = Rcpp::as<unsigned long int>(max_step_0);
-
- NumericVector out_more_0(out_more_p); 
- unsigned long int out_more = Rcpp::as<unsigned long int>(out_more_0);
- 
- CharacterVector sep_0(sep_p); 
- string sep = Rcpp::as<string>(sep_0);
- 
- NumericVector seed_0(seed_p); 
- unsigned long int seed = Rcpp::as<unsigned long int>(seed_0);
- 
- 
- //inp.b 
-    
- bool flg_var_value;
- flg_var_value=0;
- if(var_value.compare("0")!=0){
-  flg_var_value=1;
- }
- 
- bool flg_var_null;
- flg_var_null=0;
- if(var_null.compare("0")!=0){
-  flg_var_null=1;
- }
- 
- CharacterVector vy0 = Data[var_path];
- vector<string> vy = Rcpp::as<vector<string> >(vy0);
-  
- NumericVector vc0 = Data[var_conv];
- vector<unsigned long int> vc = Rcpp::as<vector<unsigned long int> >(vc0);
- 
- vector<double> vv;
- if(flg_var_value==1){
-  NumericVector vv0 = Data[var_value];
-  vv = Rcpp::as<vector<double> >(vv0);
- }
- 
- vector<unsigned long int> vn;
- if(flg_var_null==1){
-  NumericVector vn0 = Data[var_null];
-  vn = Rcpp::as<vector<unsigned long int> >(vn0);
- } 
-  
- unsigned long int i,j,k,lvy,ssize;
- unsigned long int nchannels,nchannels_sim,npassi;
- bool cfirst;
- unsigned long int start_pos,end_pos;
- string s,channel,path;
- map<string,unsigned long int> mp_channels,mp_channels_sim;
- map<unsigned long int,unsigned long int> mp_npassi;
- vector<unsigned long int> vnpassi;
-    
- lvy=(unsigned long int) vy.size();
-   
- //////////////////////
- //CODIFICA DA ONE STEP 
- //////////////////////
-    
- //mappa dei conversion value
- unsigned long int l_vui=0;
- map<double,unsigned long int> mp_vui;
- vector<double> v_vui;
- double vui;
-
- vector<string> rchannels;
- unsigned long int lrchannels,j0,z; 
- string channel_j;
- 
- vector<long int> vchannels_sim_id(order);
- map<unsigned long int, vector<long int>> mp_channels_sim_id;
- 
- nchannels=0;
- nchannels_sim=0;
- 
- vector<string> vy2(lvy);
- 
- mp_channels["(start)"]=0;
- vector<string> vchannels;
- vchannels.push_back("(start)");	 
- ++nchannels;
-
- vector<string> vchannels_sim;
- for(z=0;z<order;z++){
-  vchannels_sim_id[z]=-1;
- }
- if(order>1){
-  mp_channels_sim["(start)"]=nchannels_sim;
-  vchannels_sim.push_back("(start)");
-  vchannels_sim_id[0]=nchannels_sim;
-  mp_channels_sim_id[nchannels_sim]=vchannels_sim_id;
-  ++nchannels_sim;
- } 
- 
-
- //definizione mappa conversion value
- if(flg_var_value==1){
-  for(i=0;i<lvy;i++){
-   if(vc[i]>0){
-	vui=vv[i]/vc[i];
-    if(mp_vui.find(vui)==mp_vui.end()){
-     mp_vui[vui]=l_vui;
-     v_vui.push_back(vui);
-     ++l_vui;	
-    }
-   }
-  }
- }
- 
- for(i=0;i<lvy;i++){
-	   
-  s=vy[i];
-  s+=sep[0];
-  ssize=(unsigned long int) s.size();
-  channel="";
-  path="";
-  j=0;
-  npassi=0;
-  rchannels.clear();
-  start_pos=0;
-  end_pos=0;  
-   
-  //medium.touch
-  
-  while(j<ssize){  
-         
-   cfirst=1;
-   while(s[j]!=sep[0]){
-	if(cfirst==0){   
-     if(s[j]!=' '){
-	  end_pos=j;	 
-	 }
-    }else if((cfirst==1) & (s[j]!=' ')){
-	 cfirst=0;
-	 start_pos=j;
-	 end_pos=j;
-	}
-    ++j;     
-   }
-   
-   if(cfirst==0){
-    channel=s.substr(start_pos,(end_pos-start_pos+1));
-   
-    if(mp_channels.find(channel) == mp_channels.end()){
-     mp_channels[channel]=nchannels;
-     vchannels.push_back(channel);
-     ++nchannels;
-    }
-    
-    if(order==1){
-	  
-     if(npassi==0){
-      path="0 ";
-     }else{
-      path+=" ";
-     }
-      
-     path+=to_string(mp_channels[channel]);
-     ++npassi;  	
-    
-    }else{
-	 
-     rchannels.push_back(channel);   
-	 
-    }
-       
-   }//if end_pos
-   
-   channel="";
-   ++j;
-   
-  }//end while channel
-      	  
-  if(order>1){
-		
-	lrchannels=rchannels.size();
-	for(z=0;z<order;z++){
-	 vchannels_sim_id[z]=-1;
-	}
-	
-    if(lrchannels>(order-1)){
-		
-     npassi=lrchannels-order+1;		
-   
-     for(k=0;k<npassi;k++){
-      
-	  channel="";
-	  channel_j="";
-	 
-  	  z=0;
-	  j0=k+order;
-	  for(j=k;j<j0;j++){
-	    channel_j=rchannels[j];
-	    channel+=channel_j;
-	    vchannels_sim_id[z]=mp_channels[channel_j];
-	    ++z;
-	    if(j<(j0-1)){
-	     channel+=",";
-	    }
-	  }
-          
-	  if(mp_channels_sim.find(channel) == mp_channels_sim.end()){
-	   mp_channels_sim[channel]=nchannels_sim;
-       vchannels_sim.push_back(channel); //lo utilizzo per output more
-	   mp_channels_sim_id[nchannels_sim]=vchannels_sim_id;
-       ++nchannels_sim;
-      }
-	  
-	  path+=to_string(mp_channels_sim[channel]);
-	  path+=" ";
-	 
-	 }//end for k
-	
-
-	}else{
-				
-	  npassi=1;	
-				
-	  channel="";
-	  channel_j="";
-	  for(j=0;j<lrchannels;j++){
-	   channel_j=rchannels[j];
-	   channel+=channel_j;
-	   vchannels_sim_id[j]=mp_channels[channel_j];
-	   if(j<(lrchannels-1)){
-	     channel+=",";
-	   }
-	  }
-	    	      		  
-	  if(mp_channels_sim.find(channel) == mp_channels_sim.end()){
-	   mp_channels_sim[channel]=nchannels_sim;
-       vchannels_sim.push_back(channel); //lo utilizzo per output more
-	   mp_channels_sim_id[nchannels_sim]=vchannels_sim_id;
-       ++nchannels_sim;
-      }
-	 
-      path+=to_string(mp_channels_sim[channel]);
-	  path+=" ";
-	 	 
-	}//end else	
-	
-    path="0 "+path;	
-   
-  }else{//end order > 1
-    
-	path+=" ";
-  
-  }
-  
-  vy2[i]=path+"e"; //aggiungo lo stato finale
-  ++npassi;
- 
- }//end for
-      
- mp_channels["(conversion)"]=nchannels; //aggiungo canale conversion
- ++nchannels;
- vchannels.push_back("(conversion)");	 
-
- mp_channels["(null)"]=nchannels;
- ++nchannels;
- vchannels.push_back("(null)");	 
- 
- if(order>1){
-  mp_channels_sim["(conversion)"]=nchannels_sim;
-  vchannels_sim.push_back("(conversion)");	
-  for(z=0;z<order;z++){
-   vchannels_sim_id[0]=nchannels_sim;
-  }
-  mp_channels_sim_id[nchannels_sim]=vchannels_sim_id;
-  ++nchannels_sim;
-  
-  mp_channels_sim["(null)"]=nchannels_sim;
-  vchannels_sim.push_back("(null)");	 
-  for(z=0;z<order;z++){
-   vchannels_sim_id[0]=nchannels_sim;
-  }
-  mp_channels_sim_id[nchannels_sim]=vchannels_sim_id;
-  ++nchannels_sim;
-  
- }
- 
- if(order==1){
-  nchannels_sim=nchannels;
- }
-   
- /////////////////////////////////////////////////////
- //CREAZIONE DELLE MATRICI FUNZIONALI ALLE SIMULAZIONI
- ////////////////////////////////////////////////////
-
- unsigned long int ichannel,ichannel_old,vpi,vci,vni; 
- string channel_old;
- 
- npassi=0;
- 
- Fx S(nchannels_sim,nchannels_sim);
-  
- Fx fV(nchannels_sim,l_vui);
- 
- unsigned long int max_npassi; 
- max_npassi=0; 
-   
- for(i=0;i<lvy;i++){
-	 	 	 			 
-  s=vy2[i];
-  s+=" ";
-  ssize= (unsigned long int) s.size();
-  
-  channel="";
-  channel_old="";
-  ichannel_old=0;
-  ichannel=0;
- 
-  j=0;
-  
-  npassi=0;
-  
-  vci=vc[i];
-  if(flg_var_null==1){
-   vni=vn[i];
-  }else{
-   vni=0;
-  }	  
-  vpi=vci+vni;
-   
-  while(j<ssize){
-	  
-   while(s[j]!=' '){
-  
-    if(j<ssize){
-     channel+=s[j];
-    }
-    j=j+1;
-   }
-     
-   if(channel.compare(channel_old)!=0){
-	      
-    if(channel[0]!='0'){//se non è il channel start
-    
-     if(channel[0]=='e'){ //stato finale
-     
-	  ++npassi;
-	  
-	  if(vci>0){ //se ci sono conversion
-	   ichannel=nchannels_sim-2;
-	   S.add(ichannel_old,ichannel,vci);
-	   if(flg_var_value==1){
-		vui=vv[i]/vci;
-	    fV.add(ichannel_old,mp_vui[vui],vci);
-	   }
-	   if(vni>0){
-		goto next_null;   
-	   }else{
-		goto next_path;   
-	   }
-	  }
-	 
-	  if(vni>0){ //se non ci sono conversion
-	   next_null:;
-	   ichannel=nchannels_sim-1;
-	   S.add(ichannel_old,ichannel,vni);
-	   goto next_path;
-      }
-	 
-     }else{ //stato non finale
-	  
-	  if(vpi>0){
-       ichannel=atol(channel.c_str());
-   	   S.add(ichannel_old,ichannel,vpi);
-	  }
-    
-	 }
-	
-	 ++npassi;
-
-    }else{ //stato iniziale
-   
-     ichannel=0;
-   
-    }
-  
-    channel_old=channel;
-    ichannel_old=ichannel;
-  
-   }//end compare
-     
-   channel="";
-   
-   j=j+1;   
-   
-  }//end while j<size
-    
-  next_path:;
-  max_npassi=max(max_npassi,npassi);
-    
- }//end for 
-        
- //out matrice di transizione
- 
- List res_mtx; 
- if(out_more==1){
-  if(order==1){
-   res_mtx=S.tran_matx(vchannels);
-  }else{
-   res_mtx=S.tran_matx(vchannels_sim);
-  }
- }
- 
-   
- //f.r. transizione
- S.cum(); 
-  
- //return(0); 
-  
- //f.r. conversion value
- if(flg_var_value==1){
-  fV.cum();
- }
- 
- //distribuzione numeri uniformi
- //double iu,nuf;
- //nuf=1e6;
- //NumericVector vunif=runif(nuf);
- 
- mt19937 generator(seed);  
- uniform_real_distribution<double> distribution(0,1);
-      
- //SIMULAZIONI
-  
- unsigned long int c,c_last,nconv;
- long int id0;
- double sval0,ssval;
- vector<bool> C(nchannels);
- vector<double> T(nchannels);
- vector<double> V(nchannels);
-   
- nconv=0;
- sval0=0;
- ssval=0;
- c_last=0;
- //iu=0;
-  
- if(max_step>0){
-  max_npassi=max_step;
- }
-  
- if(nsim==0){
-  nsim=1e6;
- }
- 
- for(i=0; i<nsim; i++){
-	 	 	   
-  c=0;
-  npassi=0;
-  
-  for(k=0; k<nchannels; k++){ //svuoto il vettore del flag canali visitati
-   C[k]=0;
-  }
-
-  C[c]=1; //assegno 1 al channel start
-      
-  while(npassi<=max_npassi){ //interrompo quando raggiungo il massimo numero di passi
-   
-   //if(iu>=nuf){vunif=runif(nuf);iu=0;} //genero il canale da visitare
-   //c=S.sim(c,vunif[iu]);
-   c=S.sim(c,distribution(generator));
-   //++iu;
-   
-   if(c==nchannels_sim-2){ //se ho raggiunto lo stato conversion interrompo
-    goto go_to_conv;	
-   }else if(c==nchannels_sim-1){ //se ho raggiunto lo stato null interrompo
-	goto go_to_null;   
-   }
-   
-   if(order==1){
-	C[c]=1; //flaggo con 1 il canale visitato   
-   }else{	   
-    for(k=0; k<order; k++){
-	 id0=mp_channels_sim_id[c][k];
-	 if(id0>=0){
-      C[id0]=1;
-     }else{
-	  break;	 
-	 }
-	}
-   }
-      
-   c_last=c; //salvo il canale visitato
-   ++npassi;
- 
-  }//end while npassi 
-  
-  go_to_conv:;
- 
-  if(c==nchannels_sim-2){ //solo se ho raggiunto la conversion assegno +1 ai canali interessati (se ho raggiunto il max numero di passi è come se fossi andato a null)
-      
-   ++nconv;//incremento le conversion
-   
-   //genero per il canale c_last un valore di conversion "sval0"
-   if(flg_var_value==1){
-    //if(iu>=nuf){vunif=runif(nuf);iu=0;} 
-    //sval0=v_vui[fV.sim(c_last,vunif[iu])];
-    sval0=v_vui[fV.sim(c_last,distribution(generator))];
-    //++iu;
-   }   
-   
-   ssval=ssval+sval0;
-     
-   for(k=0; k<nchannels; k++){
-    if(C[k]==1){
-	 T[k]=T[k]+1;
-	 if(flg_var_value==1){
-	  V[k]=V[k]+sval0;
-	 }
-    }
-   }
- 
-  }//end if conv
-  
-  go_to_null:; 
- 	
- }//end for i
- 
-   
- T[0]=0; //pongo channel start = 0
- unsigned long int nch0; 
- nch0=nchannels-3;
- T[nchannels-2]=0; //pongo channel conversion = 0 
- T[nchannels-1]=0; //pongo channel null = 0 
-  
- double sn=0;
- for(i=0;i<lvy; i++){
-  sn=sn+vc[i];
- }
-  
- double sm=0;
- for(i=0;i<nchannels-1; i++){
-  sm=sm+T[i];
- }
- 
- vector<double> TV(nch0,0);
- vector<double> rTV(nch0,0);
-  
- for (k=1; k<(nch0+1); k++){
-  if(sm>0){
-   TV[k-1]=(T[k]/sm)*sn;
-   if(out_more==1){rTV[k-1]=T[k]/nconv;} //removal effects
-  }
- }
-  
-  
- vector<double> VV(nch0,0);
- vector<double> rVV(nch0,0); 
-  
- if(flg_var_value==1){
-  
-  V[0]=0; //pongo channel start = 0
-  V[nchannels-2]=0; //pongo channel conversion = 0 
-  V[nchannels-1]=0; //pongo channel null = 0 
-    
-  sn=0;
-  for(i=0;i<lvy; i++){
-   sn=sn+vv[i];
-  }
-  
-  sm=0;
-  for(i=0;i<nchannels-1; i++){
-   sm=sm+V[i];
-  }
-    
-  for(k=1; k<(nch0+1); k++){
-   if(sm>0){
-    VV[k-1]=(V[k]/sm)*sn;
-    if(out_more==1){rVV[k-1]=V[k]/ssval;} //removal effects
-   }
-  }
-     
- }
- 
- vector<string> vchannels0(nch0);
- for(k=1; k<(nch0+1); k++){
-  vchannels0[k-1]=vchannels[k];
- }
-  
- if(flg_var_value==1){ 
- 
-  if(out_more==0){
-  
-   return List::create(Named("channel_name")=vchannels0, Named("total_conversion") = TV, Named("total_conversion_value") = VV );
-  
-  }else{
-   
-   List res1=List::create(Named("channel_name")=vchannels0, Named("total_conversions") = TV, Named("total_conversion_value") = VV );
-   List res3=List::create(Named("channel_name")=vchannels0, Named("removal_effects_conversion") = rTV, Named("removal_effects_conversion_value") = rVV );
-   return List::create(Named("result") = res1, Named("transition_matrix")=res_mtx, Named("removal_effects") = res3);
-  
-  } 
- 
- }else{
-	 
-  if(out_more==0){
-  
-   return List::create(Named("channel_name")=vchannels0, Named("total_conversions") = TV);
-  
-  }else{
-   
-   List res1=List::create(Named("channel_name")=vchannels0, Named("total_conversions") = TV);
-   List res3=List::create(Named("channel_name")=vchannels0, Named("removal_effects") = rTV);
-   return List::create(Named("result") = res1, Named("transition_matrix")=res_mtx, Named("removal_effects") = res3);
-  
-  } 
-	
- }
- 
- END_RCPP
-
-}	
-
 
 vector<unsigned long int> bounds(unsigned long int parts, unsigned long int mem) {
     vector<unsigned long int>bnd;
@@ -1549,7 +921,7 @@ RcppExport SEXP choose_order_cpp(SEXP Data_p, SEXP var_path_p, SEXP var_conv_p,
  
 } 	
 
-void W_markov_model_mp_1(unsigned long int seed, vector<double> v_vui, unsigned long int lvy, vector<unsigned long int> vc, unsigned long int nch0, vector<double> vv, unsigned long int nfold, unsigned long int nsim_start,  map< unsigned long int, vector<long int> > mp_channels_sim_inv, unsigned long int max_npassi, unsigned long int nchannels_sim, unsigned long int order, bool flg_var_value, unsigned long int nchannels, Fx S, Fx fV, vector<unsigned long int> &nconv, vector<double> &ssval, vector< vector<double> > &T, vector< vector<double> > &TV, vector< vector<double> > &V, vector< vector<double> > &VV, vector<double> &v_inc_path, unsigned long int from_W, unsigned long int to_W)
+void W_markov_model_1(unsigned long int seed, vector<double> v_vui, unsigned long int lvy, vector<unsigned long int> vc, unsigned long int nch0, vector<double> vv, unsigned long int nfold, unsigned long int nsim_start,  map< unsigned long int, vector<long int> > mp_channels_sim_inv, unsigned long int max_npassi, unsigned long int nchannels_sim, unsigned long int order, bool flg_var_value, unsigned long int nchannels, Fx S, Fx fV, vector<unsigned long int> &nconv, vector<double> &ssval, vector< vector<double> > &T, vector< vector<double> > &TV, vector< vector<double> > &V, vector< vector<double> > &VV, vector<double> &v_inc_path, unsigned long int from_W, unsigned long int to_W)
 {
  
  long int id0;
@@ -1701,7 +1073,7 @@ string f_print_perc(double num){
 }
 
 
-RcppExport SEXP markov_model_mp_cpp(SEXP Data_p, SEXP var_path_p, SEXP var_conv_p, SEXP var_value_p, SEXP var_null_p, SEXP order_p, SEXP nsim_start_p, SEXP max_step_p, SEXP out_more_p, SEXP sep_p, SEXP ncore_p, SEXP nfold_p, SEXP seed_p, SEXP conv_par_p, SEXP rate_step_sim_p, SEXP verbose_p)
+RcppExport SEXP markov_model_cpp(SEXP Data_p, SEXP var_path_p, SEXP var_conv_p, SEXP var_value_p, SEXP var_null_p, SEXP order_p, SEXP nsim_start_p, SEXP max_step_p, SEXP out_more_p, SEXP sep_p, SEXP ncore_p, SEXP nfold_p, SEXP seed_p, SEXP conv_par_p, SEXP rate_step_sim_p, SEXP verbose_p)
 { 
 	 	
  BEGIN_RCPP
@@ -2167,7 +1539,7 @@ RcppExport SEXP markov_model_mp_cpp(SEXP Data_p, SEXP var_path_p, SEXP var_conv_
 
   if(ncore==1){
 	  
-	W_markov_model_mp_1(seed, v_vui, lvy, vc, nch0, vv, nfold, nsim_start, mp_channels_sim_inv, max_npassi, nchannels_sim, order, flg_var_value, nchannels, S, fV, ref(nconv), ref(ssval), ref(T), ref(TV), ref(V), ref(VV), ref(v_inc_path), limits[0], limits[1]);
+	W_markov_model_1(seed, v_vui, lvy, vc, nch0, vv, nfold, nsim_start, mp_channels_sim_inv, max_npassi, nchannels_sim, order, flg_var_value, nchannels, S, fV, ref(nconv), ref(ssval), ref(T), ref(TV), ref(V), ref(VV), ref(v_inc_path), limits[0], limits[1]);
 	  
   }else{
   
@@ -2175,7 +1547,7 @@ RcppExport SEXP markov_model_mp_cpp(SEXP Data_p, SEXP var_path_p, SEXP var_conv_
       
    //Launch ncore threads:	  
    for(unsigned long int td=0; td<ncore; td++){
-    threads[td]=thread(W_markov_model_mp_1, seed, v_vui, lvy, vc, nch0, vv, nfold, nsim_start, mp_channels_sim_inv, max_npassi, nchannels_sim, order, flg_var_value, nchannels, S, fV, ref(nconv), ref(ssval), ref(T), ref(TV), ref(V), ref(VV), ref(v_inc_path), limits[td], limits[td+1]);
+    threads[td]=thread(W_markov_model_1, seed, v_vui, lvy, vc, nch0, vv, nfold, nsim_start, mp_channels_sim_inv, max_npassi, nchannels_sim, order, flg_var_value, nchannels, S, fV, ref(nconv), ref(ssval), ref(T), ref(TV), ref(V), ref(VV), ref(v_inc_path), limits[td], limits[td+1]);
    }
       
    //Join the threads with the main thread
